// ZX81 Scrolling Mountains Demo
// by RobertK (RobertK@psc.at), 2018-02-17
// Using ASM functions written by A. Rea (see zx81plot.h)
// Compile with z88dk

#include <stdio.h>
#include <zx81.h>
#include <zx81plot.h>

// Mountain terrain limits
// Note: we use the standard ZX Basic plot coordinates with y at the bottom
// the functions in zx81plot.h have y at the top, so we will simply make 
// every call to these functions with 43-y instead of y.
#define MOUNTAIN_BASE       24
#define MOUNTAIN_LINE_MIN   27
#define MOUNTAIN_LINE_MAX   31

main ()
{

  int x,y;
  int offset;
  int xplusoffset;
  int heights[64];  // contains the height level of all x coordinates
  int bExit;
  
  bExit=0;

  // generate mountain terrain
  // With ZX81 lo-res graphics (64x48): 
  // mountain bottom at y=24, mountain ranges between 27 and 31, we start at 28
   
  y=MOUNTAIN_LINE_MIN+1;	// we start a little above the low limit
  heights[0]=y;				// set the height for x=0

  for (x = 1; x <= 63; ++x) // loop through the remaining x coordinates
  {
	// generate the height for this x value
	
	// height exceeding the top limit?
	while(y>MOUNTAIN_LINE_MAX) y--;	// decrease it until the top limit has been reached
	
	// height exceeding the lower limit?
	while(y<MOUNTAIN_LINE_MIN) y++;  // increase it until the low limit has been reached

	randomize();
	
	if ((rand() % 2)==0)	// random value 0 or 1
	{
		// mountain line goes down unless we are at the 
		// low limit (in this case we let the line go up) 
		if (y>MOUNTAIN_LINE_MIN) y--; else y++;
	}
	else
	{
		// mountain line goes up unless we are at the 
		// top limit (in this case we let the line go down) 
		if (y<MOUNTAIN_LINE_MAX) y++; else y--;
	}
	heights[x]=y;	// set the height for the current x value

  }
  
  // fill entire area from y=MOUNTAIN_BASE to MOUNTAIN_LINE_MIN-1
  // this area won't change during scrolling
  for (x = 0; x <= 63; x++)
	for (y = MOUNTAIN_BASE; y <= MOUNTAIN_LINE_MIN-1; y++)
		zx_plot(x, 43-y, 1);

  offset=0;

  while (bExit==0)		// repeat until a key has been pressed
  {  
	  // draw the top part of the mountains
	  for (x = 0; x <= 63; x++)
	  {
		  xplusoffset=x+offset;		
		  if (xplusoffset>63) xplusoffset=xplusoffset-64;
  
		  for (y = MOUNTAIN_LINE_MIN; y <= heights[xplusoffset]; y++)
		  {
			zx_plot(x, 43-y, 1);
		  }

		  // unplot the remaining upper part of the line up to 
		  // the mountain top to remove anything left over from 
		  // the previously displayed line
		  if (heights[xplusoffset]<MOUNTAIN_LINE_MAX)  // no further action needed if we are at peak level
		  {
			  for (y = heights[xplusoffset]+1; y <= MOUNTAIN_LINE_MAX; y++)
			  {
				zx_plot(x, 43-y, 0);
			  }
		  }	  
	  }	
	  
	  offset++;
	  if (offset>63) offset=0;
	  
	  if (in_Inkey()!=0) bExit=1;	// check if a key has been pressed
  }
}


